<?php

namespace Drupal\email_content_templates\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\email_content_templates\EmailContentTemplatesInfoPluginManager;
use Drupal\email_content_templates\Entity\EmailContentTemplateType;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the email content template entity edit forms.
 */
class EmailContentTemplateForm extends ContentEntityForm {

  /**
   * The email info plugin manager.
   *
   * @var \Drupal\email_content_templates\EmailContentTemplatesInfoPluginManager
   */
  private $emailInfoManager;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('date.formatter'),
      $container->get('plugin.manager.email_content_templates_info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityRepositoryInterface $entity_repository, EntityTypeBundleInfoInterface $entity_type_bundle_info, TimeInterface $time, DateFormatterInterface $date_formatter, EmailContentTemplatesInfoPluginManager $email_info_manager) {
    parent::__construct($entity_repository, $entity_type_bundle_info, $time);
    $this->emailInfoManager = $email_info_manager;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    $entity = $this->getEntity();
    $result = $entity->save();

    $message_arguments = ['%label' => $this->entity->label()];
    $logger_arguments = $message_arguments;

    if ($result == SAVED_NEW) {
      $this->messenger()
        ->addStatus($this->t('New email content template %label has been created.', $message_arguments));
      $this->logger('email_content_templates')
        ->notice('Created new email content template %label', $logger_arguments);
    }
    else {
      $this->messenger()
        ->addStatus($this->t('The email content template %label has been updated.', $message_arguments));
      $this->logger('email_content_templates')
        ->notice('Updated new email content template %label.', $logger_arguments);
    }

    $form_state->setRedirect('entity.email_content_template.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityFromRouteMatch(RouteMatchInterface $route_match, $entity_type_id) {
    if ($route_match->getRawParameter($entity_type_id) !== NULL) {
      $entity = $route_match->getParameter($entity_type_id);
    }
    else {
      $values = [];
      // If the entity has bundles, fetch it from the route match.
      $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
      if ($bundle_key = $entity_type->getKey('bundle')) {
        if (($bundle_entity_type_id = $entity_type->getBundleEntityType()) && $route_match->getRawParameter($bundle_entity_type_id)) {
          $values[$bundle_key] = $route_match->getParameter($bundle_entity_type_id)->id();
        }
        elseif ($route_match->getRawParameter($bundle_key)) {
          $values[$bundle_key] = $route_match->getParameter($bundle_key);
        }
      }
      /** @var \Drupal\email_content_templates\Entity\EmailContentTemplateType $entity_type */
      $bundle = $route_match->getParameter($bundle_entity_type_id);
      if ($bundle instanceof EmailContentTemplateType) {
        $values['subject'] = $bundle->getDefaultSubject();
        $values['body']['value'] = $bundle->getDefaultBody();
        $values['body']['format'] = $bundle->getDefaultBodyFormat();
      }

      $entity = $this->entityTypeManager->getStorage($entity_type_id)->create($values);
    }

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;
    $form['meta'] = [
      '#type' => 'details',
      '#group' => 'advanced',
      '#weight' => -10,
      '#title' => $this->t('Status'),
      '#attributes' => ['class' => ['entity-meta__header']],
      '#tree' => TRUE,
    ];
    $form['meta']['published'] = [
      '#weight' => -120,
      '#type' => 'item',
      '#markup' => $entity->isPublished() ? $this->t('Published') : $this->t('Not published'),
      '#access' => !$entity->isNew(),
      '#wrapper_attributes' => ['class' => ['entity-meta__title']],
    ];
    $form['meta']['changed'] = [
      '#type' => 'item',
      '#title' => $this->t('Last saved'),
      '#weight' => -110,
      '#markup' => !$entity->isNew() ? $this->dateFormatter->format($entity->getChangedTime(), 'short') : $this->t('Not saved yet'),
      '#wrapper_attributes' => ['class' => ['entity-meta__last-saved']],
    ];
    $form['meta']['author'] = [
      '#type' => 'item',
      '#weight' => -100,
      '#title' => $this->t('Author'),
      '#markup' => $entity->getOwner()->getAccountName(),
      '#wrapper_attributes' => ['class' => ['entity-meta__author']],
    ];
    if (isset($form['uid'])) {
      $form['author'] = [
        '#type' => 'details',
        '#title' => $this->t('Authoring information'),
        '#group' => 'advanced',
        '#weight' => 90,
        '#optional' => TRUE,
      ];
      $form['uid']['#group'] = 'author';
      if (isset($form['created'])) {
        $form['created']['#group'] = 'author';
      }
    }
    if ($this->moduleHandler->moduleExists('token')) {
      $email_plugin = $this->entity->getEmailPlugin();
      if (!empty($email_plugin) && $email_plugin !== '_none') {
        /** @var \Drupal\email_content_templates\EmailContentTemplatesInfoInterface $plugin */
        $plugin = $this->emailInfoManager->createInstance($email_plugin);
        $token_map = $plugin->paramTokenMap();
        if (!empty($token_map)) {
          $form['body']['token_link'] = [
            '#theme' => 'token_tree_link',
            '#token_types' => array_values($token_map),
            '#global_types' => TRUE,
          ];
        }
      }
    }
    return $form;
  }

}
