<?php

namespace Drupal\email_content_templates\Form;

use Drupal\Core\Entity\BundleEntityFormBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\email_content_templates\EmailContentTemplatesInfoPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for emailcontenttemplate type forms.
 */
class EmailContentTemplateTypeForm extends BundleEntityFormBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.email_content_templates_info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(EmailContentTemplatesInfoPluginManager $email_info_manager) {
    $this->emailInfoManager = $email_info_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $entity_type = $this->entity;
    if ($this->operation == 'add') {
      $form['#title'] = $this->t('Add emailcontenttemplate type');
    }
    else {
      $form['#title'] = $this->t(
        'Edit %label template type',
        ['%label' => $entity_type->label()]
      );
    }

    $form['label'] = [
      '#title' => $this->t('Label'),
      '#type' => 'textfield',
      '#default_value' => $entity_type->label(),
      '#description' => $this->t('The human-readable name of this emailcontenttemplate type.'),
      '#required' => TRUE,
      '#size' => 30,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $entity_type->getDescription(),
      '#description' => $this->t("Template type description."),
      '#required' => TRUE,
    ];

    $form['default_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Subject'),
      '#maxlength' => 255,
      '#default_value' => $entity_type->getDefaultSubject(),
      '#description' => $this->t("Default subject."),
      '#required' => FALSE,
    ];

    $form['default_body'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Default Body'),
      '#description' => $this->t("Default Body."),
      '#required' => FALSE,
      '#default_value' => $entity_type->getDefaultBody(),
      '#format' => $entity_type->getDefaultBodyFormat(),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity_type->id(),
      '#maxlength' => EntityTypeInterface::BUNDLE_MAX_LENGTH,
      '#machine_name' => [
        'exists' => [
          'Drupal\email_content_templates\Entity\EmailContentTemplateType',
          'load',
        ],
        'source' => ['label'],
      ],
      '#description' => $this->t('A unique machine-readable name for this emailcontenttemplate type. It must only contain lowercase letters, numbers, and underscores.'),
    ];

    $definitions = $this->emailInfoManager->getDefinitions();

    $options = [
      '_none' => $this->t('- Please choose -'),
    ];
    foreach ($definitions as $definition) {
      $options[$definition['id']] = $definition['label'];
      $form["email_plugin"] = [
        '#type' => 'select',
        '#title' => $this->t('Email plugin'),
        '#options' => $options,
      ];
      if (!empty($this->entity)) {
        $form['email_plugin']['#default_value'] = $this->entity->getEmailPlugin();
      }
    }
    return $this->protectBundleIdElement($form);
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Save template type');
    $actions['delete']['#value'] = $this->t('Delete template type');
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity_type = $this->entity;

    $entity_type->set('id', trim($entity_type->id()));
    $entity_type->set('label', trim($entity_type->label()));
    $entity_type->setDescription(trim($form_state->getValue('description')));
    $entity_type->setDefaultSubject(trim($form_state->getValue('default_subject')));
    $entity_type->setDefaultBody(trim($form_state->getValue('default_body')['value'] ?? ''));
    $entity_type->setDefaultBodyFormat(trim($form_state->getValue('default_body')['format'] ?? ''));
    $entity_type->setEmailPlugin($form_state->getValue('email_plugin'));

    $status = $entity_type->save();

    $t_args = ['%name' => $entity_type->label()];
    if ($status == SAVED_UPDATED) {
      $message = $this->t('The template type %name has been updated.', $t_args);
    }
    elseif ($status == SAVED_NEW) {
      $message = $this->t('The template type %name has been added.', $t_args);
    }
    $this->messenger()->addStatus($message);

    $form_state->setRedirectUrl($entity_type->toUrl('collection'));
  }

}
